#!/usr/bin/env python3
"""Customer Tests"""

import clover
from clover import Customer, Token
from clover.test import MockedTestCase
from clover.test.constants import CARD
from clover.test.mock_responses import (
    CREATE_TOKEN,
    CREATE_CUSTOMER,
    MODIFY_CUSTOMER,
    DELETE_SOURCE_CUSTOMER
)

class TestCharge(MockedTestCase):
    
    def test_create_customer(self):
        """Tests creating a customer."""

        self.set_mock_json(CREATE_TOKEN)

        token = token = Token.create(
            card=CARD
        )

        self.set_mock_json(CREATE_CUSTOMER)

        response = Customer.create(
            email="test@clover.com",
            source=token['id']
        )

        # Check that it is a customer.
        self.assertKeyValueEqual(response, 'object', 'customer')

    def test_modify_customer(self):
        """Tests modifying a customer."""

        self.set_mock_json(CREATE_TOKEN)

        token = token = Token.create(
            card=CARD
        )

        self.set_mock_json(CREATE_CUSTOMER)

        response = Customer.create(
            email="test@clover.com",
            source=token['id']
        )

        self.set_mock_json(MODIFY_CUSTOMER)

        response = Customer.modify(
            sid=response['id'],
            email="test@clover.com",
            source=token['id'],
            first_name="clover",
            last_name="test"
        )

        # Check that it is a customer.
        self.assertKeyValueEqual(response, 'object', 'customer')

        # Check that first name is updated.
        self.assertKeyValueEqual(response, 'firstName', 'clover')

        # Check that last name is updated.
        self.assertKeyValueEqual(response, 'lastName', 'test')

    def test_delete_source_customer(self):
        """Tests deleting a payment method of a customer."""

        self.set_mock_json(CREATE_TOKEN)

        token = token = Token.create(
            card=CARD
        )


        self.set_mock_json(CREATE_CUSTOMER)

        create_response = Customer.create(
            email="test@clover.com",
            source=token['id']
        )

        # Assumes only one source.
        source_id = create_response['sources']['data']['elements'][0]

        self.set_mock_json(DELETE_SOURCE_CUSTOMER)

        delete_response = Customer.delete_source(
            customer_id=create_response['id'],
            source_id=source_id
        )

        # Check that it is a deletecard.
        self.assertKeyValueEqual(delete_response, 'object', 'deletedcard')

        # Check that id matches.
        self.assertKeyValueEqual(delete_response, 'id', source_id)

        # Check that it is deleted.
        self.assertKeyValueEqual(delete_response, 'deleted', True)



#!/usr/bin/env python3
"""Order"""

from clover.abstract import (
    CreatableResource,
    RetrievableResource,
    ListableResource,
    ModifiableResource,
    add_resource_method
)

@add_resource_method(name='pay', http_verb='post')
@add_resource_method(name='return_order', http_verb='post', http_path='returns')
class Order(CreatableResource, RetrievableResource, ListableResource, ModifiableResource):
    
    path = 'orders'



#!/usr/bin/env python3
"""Listable Resource"""

from clover import BaseRequestor


class ListableResource(BaseRequestor):

    @classmethod
    def list(cls, **params):
        """Retrieves a list of objects."""

        return cls.request(
            method='get',
            params=params
        )



#!/usr/bin/env python3
"""Modifiable Resource"""

from clover import BaseRequestor


class ModifiableResource(BaseRequestor):

    @classmethod
    def modify(cls, sid, **params):
        """Modifies an object."""

        return cls.request(
            method='put',
            path=[sid],
            json=params
        )



#!/usr/bin/env python3
"""Order Tests"""

from clover import Order
from clover.test import MockedTestCase
from clover.utils import json_ps
from clover.test.mock_responses import (
    CREATE_ORDER,
    RETRIEVE_ORDER,
    LIST_ORDER,
    MODIFY_ORDER,
    CREATE_TOKEN,
    PAY_ORDER,
    CAPTURE_ORDER,
    RETURN_ORDER
)


class TestOrder(MockedTestCase):
    
    def test_create_order(self):
        """Tests creating a order."""

        self.set_mock_json(CREATE_ORDER)
 
        order = Order.create(
            currency="usd",
            items=[
                {
                    "amount": 100,
                    "currency": "string",
                    "description": "string",
                    "parent": "string",
                    "quantity": 1,
                    "type": "discount"
                }
            ]
        )

        self.assertKeyValueEqual(order, 'status', 'created')

    def test_retrieve_order(self):
        """Tests to retrieving a order."""
        self.set_mock_json(RETRIEVE_ORDER)

        order = CREATE_ORDER

        r_order = Order.retrieve(
            id=order['id']
        )

        # Check that it isn't an error code.
        self.assertNotInDict('error', r_order, r_order)

        # Check that it is the same order object.
        self.assertEqual(r_order['id'], order['id'], json_ps(r_order))

    def test_list_order(self):
        """Tests listing orders."""
        self.set_mock_json(LIST_ORDER)

        num_orders = 3

        orders = Order.list(limit=num_orders)

        # Check that it isn't an error code.
        self.assertNotInDict('error', orders, orders)

        # Check that it is a list object.
        self.assertKeyValueEqual(orders, 'object', 'list')

        # Check that the correct amount was listed.
        self.assertEqual(
            len(orders['data']['elements']),
            num_orders,
            json_ps(orders)
        )

    def est_pay_order(self):
        """Tests paying an order."""

        self.set_mock_json(PAY_ORDER)

        order = CREATE_ORDER
        token = CREATE_TOKEN

        r_order = Order.pay(
            sid=order['id'],
            source=token['id']
        )

        # Check that it isn't an error code.
        self.assertNotInDict('error', r_order, r_order)

    def est_capture_order(self):
        """Tests capturing an order."""

        self.set_mock_json(CAPTURE_ORDER)

        order = CREATE_ORDER

        r_order = Order.capture(
            sid=order['id']
        )

        # Check that it isn't an error code.
        self.assertNotInDict('error', r_order, r_order)

    def est_return_order(self):
        """Tests returning an order."""

        self.set_mock_json(RETURN_ORDER)

        order = CREATE_ORDER

        r_order = Order.return_order(
            sid=order['id']
        )

        # Check that it isn't an error code.
        self.assertNotInDict('error', r_order, r_order)



"""Resource Method Decorator

Creates methods that hit endpoints of the form: /v1/{resource}/{id}/name
"""


def add_resource_method(name, http_verb, http_path=None):
    """Class decorator to add a method."""

    # Default path is the method name.
    if http_path is None:
        http_path = name

    def wrapper(cls):

        # Define the method.
        def resource_request(cls, sid, **params):

            json_body = None
            query_params = None


            if http_verb.lower() in set(['post', 'put']):
                # Set as JSON in body.
                json_body = params

                return cls.request(
                    method=http_verb,
                    path=[sid, http_path],
                    json=json_body
                )
            else:
                # Set as url query parameters.
                query_params = params
                return cls.request(
                    method=http_verb,
                    path=[sid, http_path],
                    params=query_params
                )

        # Add the method to the class.
        setattr(cls, name, classmethod(resource_request))
        
        return cls

    return wrapper


#!/usr/bin/env python3
"""Deletable Resource"""

from clover import BaseRequestor


class DeletableResource(BaseRequestor):

    @classmethod
    def delete(cls, sid):
        """Modifies an object."""

        return cls.request(
            method='delete',
            path=[sid]
        )



#!/usr/bin/env python3
"""Customer"""

from clover.abstract import (
    CreatableResource,
    ModifiableResource
)


class Customer(CreatableResource, ModifiableResource):

    path = 'customers'

    @classmethod
    def delete_source(cls, customer_id, source_id):
        """Deletes a payment method from a customer."""

        return cls.request(
            method='delete',
            path=[customer_id, 'sources', source_id]
        )



RETRIEVE_PUBLIC_ACCESS_KEY = {
    "apiAccessKey": "309f30e7fbacd10d6af9f7a38fce5b16",
    "merchantUuid": "CVSWF4EE5DP81",
    "developerAppUuid": "1HXD300CFCJ8C",
    "active": True,
    "createdTime": 1565729754000,
    "modifiedTime": 1565729754000
}

CREATE_TOKEN = {
    "id": "clv_1TSTS89P1vANcEi3DXaQEck7",
    "object": "token",
    "card": {
        "exp_month": "12",
        "exp_year": "2030",
        "last4": "1111",
        "brand": "VISA"
    }
}

RETRIEVE_TOKEN = {
    "card": {
        "brand": "VISA",
        "created_time": 2019,
        "encrypted_pan": "NJUWmTC+uBz4KDPMwHDw0NFvDrv8bj8EZ7Jl3u6KAW0=",
        "exp_month": "12",
        "exp_year": "2030",
        "first6": "411111",
        "id": "E46M2RK9B00JJ",
        "last4": "1111",
        "modified_time": 2019
    },
    "cloverToken": {
        "createdTime": 2019,
        "developerUuid": "mockdeveloper",
        "lastAccessedTime": 0,
        "merchantUuid": "mockmerchant",
        "tokenValue": "clv_1TSTS89P1vANcEi3DXaQEck7"
    },
    "tokenMeta": {
        "createdTime": 2019,
        "modifiedTime": 2019,
        "paymentInstrument": "CARD",
        "tokenProvider": "MOCK",
        "tokenStatus": "NEW"
    }
}

AFTER_MODIFY_RETRIEVE_TOKEN = {
    "card": {
        "brand": "VISA",
        "created_time": 2019,
        "encrypted_pan": "NJUWmTC+uBz4KDPMwHDw0NFvDrv8bj8EZ7Jl3u6KAW0=",
        "exp_month": "12",
        "exp_year": "2030",
        "first6": "411111",
        "id": "E46M2RK9B00JJ",
        "last4": "1111",
        "modified_time": 2019
    },
    "cloverToken": {
        "createdTime": 2019,
        "developerUuid": "mockdeveloper",
        "lastAccessedTime": 2019,
        "merchantUuid": "mockmerchant",
        "tokenValue": "clv_1TSTS89P1vANcEi3DXaQEck7"
    },
    "tokenMeta": {
        "createdTime": 2019,
        "modifiedTime": 2019,
        "paymentInstrument": "CARD",
        "tokenProvider": "MOCK",
        "tokenStatus": "NEW",
        "tokenType": "RECURRING"
    }
}

CREATE_CHARGE = {
    "amount": 999,
    "capture": False,
    "created": 1563825782866,
    "currency": "usd",
    "description": "Example charge",
    "id": "KW5VKX3GG90CP",
    "outcome": {
        "network_status": "approved_by_network",
        "type": "authorized"
    },
    "paid": True,
    "refunded": False,
    "source": {
        "cvc_check": "unavailable",
        "object": "clv_1TSTSutExahSCcYgnh9z7Yct"
    },
    "status": "succeeded"
}

RETRIEVE_CHARGE = {
    "amount": 999,
    "created": 1563830252722,
    "description": "Example charge",
    "id": "KW5VKX3GG90CP",
    "order": "ZCXV7QA1SSZZM",
    "outcome": {
        "network_status": "approved_by_network",
        "risk_level": "normal",
        "risk_score": 10,
        "type": "authorized"
    },
    "paid": True,
    "refunded": False,
    "source": {},
    "status": "succeeded"
}

LIST_CHARGE = {
    "data": {
        "elements": [
            {
                "amount": 999,
                "created": 1563830334562,
                "description": "Example charge",
                "id": "1RZE6CCMHSENA",
                "order": "3XVF6TWSH1XAJ",
                "outcome": {
                    "network_status": "approved_by_network",
                    "risk_level": "normal",
                    "risk_score": 10,
                    "type": "authorized"
                },
                "paid": True,
                "refunded": False,
                "source": {},
                "status": "succeeded"
            },
            {
                "amount": 999,
                "created": 1563830334581,
                "description": "Example charge",
                "id": "SGZ5C90AD8B7M",
                "order": "P91V2TDV06FJY",
                "outcome": {
                    "network_status": "approved_by_network",
                    "risk_level": "normal",
                    "risk_score": 10,
                    "type": "authorized"
                },
                "paid": True,
                "refunded": False,
                "source": {},
                "status": "succeeded"
            },
            {
                "amount": 999,
                "created": 1563830334620,
                "description": "Example charge",
                "id": "VMD7MFZ5PZYE0",
                "order": "NMW0YMGFQ3CNT",
                "outcome": {
                    "network_status": "approved_by_network",
                    "risk_level": "normal",
                    "risk_score": 10,
                    "type": "authorized"
                },
                "paid": True,
                "refunded": False,
                "source": {},
                "status": "succeeded"
            }
        ]
    },
    "has_more": True,
    "object": "list",
    "url": "/v1/charges"
}

CREATE_CUSTOMER = {
    "id": "XR01MDZ9ZB9QG",
    "object": "customer",
    "created": 1564410985777,
    "currency": "usd",
    "email": "test@clover.com",
    "sources": {
        "object": "list",
        "data": {
            "elements": [
                "MB33YF9F4RGS2"
            ]
        },
        "has_more": False
    },
    "shipping": {
        "name": "null null"
    }
}

MODIFY_CUSTOMER = {
    "id": "XR01MDZ9ZB9QG",
    "object": "customer",
    "created": 1564411087340,
    "currency": "usd",
    "email": "test@clover.com",
    "firstName": "clover",
    "lastName": "test",
    "sources": {
        "object": "list",
        "data": {
            "elements": [
                "MB33YF9F4RGS2"
            ]
        },
        "has_more": False
    }
}

DELETE_SOURCE_CUSTOMER = {
    "deleted": True,
    "id": "MB33YF9F4RGS2",
    "object": "deletedcard"
}

CREATE_REFUND = {
    "amount": 999,
    "capture": False,
    "created": 1563825782866,
    "currency": "usd",
    "description": "Example charge",
    "id": "KW5VKX3GG90CP",
    "outcome": {
        "network_status": "approved_by_network",
        "type": "authorized"
    },
    "paid": True,
    "refunded": False,
    "source": {
        "cvc_check": "unavailable",
        "object": "clv_1TSTSutExahSCcYgnh9z7Yct"
    },
    "status": "succeeded"
}

RETRIEVE_REFUND = {
    "amount": 999,
    "created": 1563830252722,
    "description": "Example charge",
    "id": "KW5VKX3GG90CP",
    "order": "ZCXV7QA1SSZZM",
    "outcome": {
        "network_status": "approved_by_network",
        "risk_level": "normal",
        "risk_score": 10,
        "type": "authorized"
    },
    "paid": True,
    "refunded": False,
    "source": {},
    "status": "succeeded"
}

LIST_REFUND = {
    "data": {
        "elements": [
            {
                "amount": 999,
                "created": 1563830334562,
                "description": "Example charge",
                "id": "1RZE6CCMHSENA",
                "order": "3XVF6TWSH1XAJ",
                "outcome": {
                    "network_status": "approved_by_network",
                    "risk_level": "normal",
                    "risk_score": 10,
                    "type": "authorized"
                },
                "paid": True,
                "refunded": False,
                "source": {},
                "status": "succeeded"
            },
            {
                "amount": 999,
                "created": 1563830334581,
                "description": "Example charge",
                "id": "SGZ5C90AD8B7M",
                "order": "P91V2TDV06FJY",
                "outcome": {
                    "network_status": "approved_by_network",
                    "risk_level": "normal",
                    "risk_score": 10,
                    "type": "authorized"
                },
                "paid": True,
                "refunded": False,
                "source": {},
                "status": "succeeded"
            },
            {
                "amount": 999,
                "created": 1563830334620,
                "description": "Example charge",
                "id": "VMD7MFZ5PZYE0",
                "order": "NMW0YMGFQ3CNT",
                "outcome": {
                    "network_status": "approved_by_network",
                    "risk_level": "normal",
                    "risk_score": 10,
                    "type": "authorized"
                },
                "paid": True,
                "refunded": False,
                "source": {},
                "status": "succeeded"
            }
        ]
    },
    "has_more": True,
    "object": "list",
    "url": "/v1/charges"
}

CREATE_ORDER = {
    "id": "4CEQ3THZJPXJA",
    "amount": 0,
    "amount_returned": 0,
    "currency": "usd",
    "created": 1565030627000,
    "items": {
        "elements": [
            {
                "parent": "string",
                "quantity": 1,
                "amount": 100,
                "currency": "string",
                "description": "string"
            }
        ]
    },
    "status": "created"
}

RETRIEVE_ORDER = {
    "id": "4CEQ3THZJPXJA",
    "amount": 0,
    "currency": "USD"
}

LIST_ORDER = {
    "object": "list",
    "url": "/v1/orders",
    "has_more": True,
    "data": {
        "elements": [
            {
                "id": "4CEQ3THZJPXJA",
                "amount": 0,
                "currency": "USD"
            },
            {
                "id": "F1ZZP6VK7NPSA",
                "currency": "USD",
                "charge": "2XAP0F7K5EVJT"
            },
            {
                "id": "59EQ70BSV53D0",
                "currency": "USD",
                "charge": "6E327M9S7JSFE"
            }
        ]
    }
}

MODIFY_ORDER = {}

PAY_ORDER = {}

CAPTURE_ORDER = {}

RETURN_ORDER = {}



from clover.abstract.creatable_resource import CreatableResource
from clover.abstract.retrievable_resource import RetrievableResource
from clover.abstract.listable_resource import ListableResource
from clover.abstract.modifiable_resource import ModifiableResource
from clover.abstract.deletable_resource import DeletableResource
from clover.abstract.add_resource_method import add_resource_method



DEFAULT_REQUEST_DELAY_SECS = 0.1


# Sandbox Credentials
SANDBOX_ACCESS_TOKEN = None
SANDBOX_API_KEY = None
SANDBOX_MERCHANTUUID = None
SANDBOX_MID = None

REQUESTS_PATH = 'clover.http_client.requests'

CARD = {
    'number': '4111111111111111',
    'brand': 'VISA',
    'exp_month': '12',
    'exp_year': '2030',
    'cvv': '123',
}



# Needed for authentication and authorization.
access_token = None
api_key = None


merchant_uuid = None
developer_app_uuid = None
mid = None

# Endpoints
# base_url = 'https://scl.clover.com'
# tokenization_base_url = 'https://token.clover.com'

# To test api in Sandbox environment, below should be updated to point to Sandbox
#Sandbox endpoints
base_url = 'https://scl-sandbox.dev.clover.com'
tokenization_base_url = 'https://token-sandbox.dev.clover.com'

# Imports
from .http_client import HTTPClient
from .base_requestor import BaseRequestor
from .resources.token import Token
from .resources.charge import Charge
from .resources.customer import Customer
from .resources.refund import Refund
from .resources.order import Order
from .resources.public_access_key_management_service import PublicAccessKey
from .utils import (
    set_api_endpoint,
    set_tokenization_api_endpoint,
    set_public_access_key_endpoint
)



#!/usr/bin/env python3
"""Refund"""

from clover.abstract import (
    CreatableResource,
    RetrievableResource,
    ListableResource
)


class Refund(CreatableResource, RetrievableResource, ListableResource):
    
    path = 'refunds'



#!/usr/bin/env python3
""" Base Requestor

Base class that wraps an endpoint.
"""

import clover
from clover import HTTPClient


class BaseRequestor(object):

    path = None

    @classmethod
    def request(cls, method, path=None, **kwargs):
        """Requests with path parameters."""

        # Append path parameters to url.
        if path is not None:
            url = cls.build_url_with_path_parameters(path)
        else:
            url = cls.get_class_url()

        response = HTTPClient.request(
            method=method,
            url=url, 
            **kwargs)
        
        return response.json()

    @classmethod
    def build_url_with_path_parameters(cls, path):
        """Builds a url with path parameters."""
        path_str = '/'.join(path)
        url_with_path = '/'.join([cls.get_class_url(), path_str])
        return url_with_path

    @classmethod
    def get_class_url(cls, version=1):
        """Builds the endpoint url."""

        url = '{base_url}/v{version}/{path}'.format(
            base_url=clover.base_url,
            version=version,
            path=cls.path
        )

        return url



#!/usr/bin/env python3
"""Test Utility functions.

Random functions that are used commonly.
"""
from time import sleep
try:
    # Python 3: Mock is built in.
    from unittest.mock import patch
except ImportError:
    # Python 2.7: Mock must be installed via pip.
    from mock import patch
from clover.test.constants import DEFAULT_REQUEST_DELAY_SECS, REQUESTS_PATH

def request_delay(seconds=DEFAULT_REQUEST_DELAY_SECS):
    """Sleep for a set period of time before the next request."""
    sleep(seconds)

def mock_request(method):
    """Mocks the requests method."""

    # Form the path to the method within in the module.
    method_path = '{base_path}.{method}'.format(
                        base_path=REQUESTS_PATH,
                        method=method
    )

    # Get the patch object.
    mock = patch(method_path)

    return mock



#!/usr/bin/env python3
"""API Test Case

Test case for an API resource.
"""
from unittest import TestCase
import clover
from clover.test.constants import (
    SANDBOX_ACCESS_TOKEN,
    SANDBOX_API_KEY,
    SANDBOX_MERCHANTUUID,
    SANDBOX_MID
)
from clover.utils import json_ps

class ApiTestCase(TestCase):
    
    @classmethod
    def setup_class(cls):
        """Starts mocks for HTTP requests."""

        # Sets the package wide access token.
        # Required when requesting the actual server.
        clover.access_token = SANDBOX_ACCESS_TOKEN

        clover.api_key = SANDBOX_API_KEY

        # Merchant information.
        clover.merchant_uuid = SANDBOX_MERCHANTUUID
        clover.mid = SANDBOX_MID

    def assertInDict(self, first, second, msg=None):
        """Checks that first is in second.
        
        Pretty prints dictionary.
        """
        self.assertIn(first, second, json_ps(msg))

    def assertNotInDict(self, first, second, msg=None):
        """Checks that first is not in second.
        
        Pretty prints dictionary.
        """
        self.assertNotIn(first, second, json_ps(msg))

    def assertKeyValueEqual(self, first, key, second):
        """Checks that key is in first and that the value is equal to second.
        
        Pretty prints dictionary.
        """
        self.assertInDict(key, first, msg=first)
        self.assertEqual(first[key], second, msg=json_ps(first))

    def assertGetValue(self, key, second):
        self.assertInDict(key, second, second)
        return second[key]

    def set_mock_json(self, data):
        """Dummy method for mocking subclass."""
        pass



#!/usr/bin/env python3
"""Refund Tests"""

from clover import Refund
from clover.test import MockedTestCase
from clover.utils import json_ps
from clover.test.mock_responses import (
    CREATE_CHARGE,
    CREATE_REFUND,
    RETRIEVE_REFUND,
    LIST_REFUND
)


class TestRefund(MockedTestCase):
    
    def test_create_refund(self):
        """Tests creating a refund."""

        charge = CREATE_CHARGE

        self.set_mock_json(CREATE_REFUND)
            
        refund = Refund.create(
            charge=charge['id']
        )

        self.assertKeyValueEqual(refund, 'status', 'succeeded')

    def test_retrieve_refund(self):
        """Tests to retrieving a refund."""
        self.set_mock_json(RETRIEVE_REFUND)

        refund = CREATE_REFUND

        r_refund = Refund.retrieve(
            id=refund['id']
        )

        # Check that it isn't an error code.
        self.assertNotInDict('error', r_refund, r_refund)

        # Check that it is the same refund object.
        self.assertEqual(r_refund['id'], refund['id'], json_ps(r_refund))

    def test_list_refund(self):
        """Tests listing refunds."""
        self.set_mock_json(LIST_REFUND)

        num_refunds = 3

        refunds = Refund.list(limit=num_refunds)

        # Check that it isn't an error code.
        self.assertNotInDict('error', refunds, refunds)

        # Check that it is a list object.
        self.assertKeyValueEqual(refunds, 'object', 'list')

        # Check that the correct amount was listed.
        self.assertEqual(
            len(refunds['data']['elements']),
            num_refunds,
            json_ps(refunds)
        )



#!/usr/bin/env python3
"""Createable Resource"""

from clover import BaseRequestor


class CreatableResource(BaseRequestor):

    @classmethod
    def create(cls, **params):
        """Creates an object."""

        return cls.request(
            method='post',
            json=params
        )



#!/usr/bin/env python3
"""Public Access Key Management Service (PAKMS)"""

import clover
from clover import BaseRequestor


class PublicAccessKey(BaseRequestor):

    path = 'pakms/apikey'

    @classmethod
    def get_class_url(cls):
        """Overrides because PAKMS uses a different server.

        Note:
            Removed version in path.
        """

        url = '{base_url}/{path}'.format(
            base_url=clover.base_url,
            path=cls.path
        )

        return url

    @classmethod
    def retrieve(cls):
        """Retrieves PAKMS credentials."""

        return cls.request(
            method='get'
        )



#!/usr/bin/env python3
"""Utility functions.

Random functions that are used commonly.
"""
import json
import clover

def json_ps(dict_obj):
    """
    Converts a dictionary object to a pretty JSON string.
    """
    pretty_dict = json.dumps(dict_obj, indent=4, sort_keys=True)
    return pretty_dict

def json_pp(dict_obj):
    """Pretty prints dictionary object.

    Args:
        dict_obj (dict): Any dictionary object.
    """
    print(json_ps(dict_obj))

def remove_all_none_values(dictionary):
    """Removes all None value keys."""
    filtered = {
        key: val for key, val in dictionary.items() if val is not None
    }
    dictionary.clear()
    dictionary.update(filtered)

def set_api_endpoint(url):
    """Sets the package wide base url."""
    clover.base_url = url

def set_tokenization_api_endpoint(url):
    """Sets the package wide tokenization base url."""
    clover.tokenization_base_url = url

def set_public_access_key_endpoint(url):
    """Sets the package wide public access key base url."""
    clover.base_url = url

def set_credentials_from_public_access_key():
    """Sets package wide api key, merchant UUID, and developer app UUID."""
    
    credentials = clover.PublicAccessKey.retrieve()

    clover.api_key = credentials['apiAccessKey']
    clover.merchant_uuid = ['merchantUuid']
    clover.developer_app_uuid = ['developerAppUuid']



#!/usr/bin/env python3
"""Retrievable Resource"""

from clover import BaseRequestor


class RetrievableResource(BaseRequestor):

    @classmethod
    def retrieve(cls, id):
        """Makes create request for an object."""

        return cls.request(
            method='get',
            path=[id]
        )




#!/usr/bin/env python3
"""Token Tests"""

from clover import Token
from clover.test import MockedTestCase
from clover.test.constants import CARD
from clover.test.mock_responses import (
    CREATE_TOKEN,
    RETRIEVE_TOKEN,
    AFTER_MODIFY_RETRIEVE_TOKEN
)


class TestToken(MockedTestCase):
    
    def test_create_token(self):
        """Trys to create a token using card details."""

        self.set_mock_json(CREATE_TOKEN)

        token = Token.create(
            card=CARD
        )

        self.assertKeyValueEqual(token, 'object', 'token')

    def test_retrieve_token(self):
        """Tests token retrieval using a token ID."""
        token = CREATE_TOKEN

        token_id = token['id']

        self.set_mock_json(RETRIEVE_TOKEN)
        response = Token.retrieve(token_id)

        # Check there's a token object.
        r_token = self.assertGetValue('cloverToken', response)

        # Make sure that it is the same token.
        self.assertKeyValueEqual(r_token, 'tokenValue', token_id)

    def test_modify_token(self):
        """Tests modifying a token to become recurring."""
        token = CREATE_TOKEN

        token_id = token['id']
        new_type = "RECURRING"

        response = Token.modify(
            sid=token['id'],
            type=new_type
        )

        self.set_mock_json(AFTER_MODIFY_RETRIEVE_TOKEN)
        response = Token.retrieve(token_id)

        # Make sure that it is the same token.
        r_token = self.assertGetValue('cloverToken', response)
        self.assertKeyValueEqual(r_token, 'tokenValue', token_id)

        # Make sure that the token type is updated.
        meta = self.assertGetValue('tokenMeta', response)
        self.assertKeyValueEqual(meta, 'tokenType', new_type)



#!/usr/bin/env python3
"""Public Access Key Management Service Tests"""

from clover import PublicAccessKey
from clover.test import MockedTestCase
from clover.test.mock_responses import (
    RETRIEVE_PUBLIC_ACCESS_KEY
)


class TestPublicAccessKey(MockedTestCase):

    def test_retrieve_public_access_key(self):
        """Tests public access key retrieval."""

        self.set_mock_json(RETRIEVE_PUBLIC_ACCESS_KEY)

        response = PublicAccessKey.retrieve()

        # Check that it has all the required credentials.
        self.assertInDict('apiAccessKey', response)
        self.assertInDict('merchantUuid', response)
        self.assertInDict('developerAppUuid', response)



#!/usr/bin/env python3
"""Token"""

import clover
from clover.abstract import (
    CreatableResource,
    RetrievableResource,
    ModifiableResource
)


class Token(CreatableResource, RetrievableResource, ModifiableResource):

    path = 'tokens'

    @classmethod
    def get_class_url(cls, version=1):
        """Overrides because Tokenization uses a different server."""

        url = '{base_url}/v{version}/{path}'.format(
            base_url=clover.tokenization_base_url,
            version=version,
            path=cls.path
        )

        return url

    @classmethod
    def request(cls, method, path=None, **kwargs):
        """Overrides because we need the api key."""

        # Add the api key into the headers just for Token.
        if 'params' in kwargs:
            headers = kwargs['headers']
        else:
            headers = {}

        headers['APIKEY'] = clover.api_key

        kwargs['headers'] = headers
        
        return super(Token, cls).request(
            method=method,
            path=path,
            **kwargs
        )



#!/usr/bin/env python3
"""Charge"""

from clover.abstract import (
    CreatableResource,
    RetrievableResource,
    ListableResource,
    add_resource_method
)

@add_resource_method(name='capture', http_verb='post')
class Charge(CreatableResource, RetrievableResource, ListableResource):
    
    path = 'charges'



#!/usr/bin/env python3
"""
Clover ECOMM Python SDK Package
"""
import setuptools

setuptools.setup(
    name='ecommClover',
    version='1.0.0',
    description="Python SDK for Clover's E-Commerce APIs.",
    url='https://github.com/clover/ecomm-SDK-python',
    author='Clover',
    author_email='support@clover.com, deepali.mittal@clover.com',
    license='MIT',
    keywords="clover api payments",
    packages=setuptools.find_packages(),
    install_requires=[
    'requests',
    ],
    test_suite='nose.collector',
    tests_require=['nose'],
    zip_safe=False,
    python_requires=">=3.0.*",
    project_urls={
        "Bug Tracker": "https://github.com/clover/ecomm-SDK-python/issues",
        "Documentation": "https://docs.clover.com",
        "Source Code": "https://github.com/clover/ecomm-SDK-python",
    }
)



#!/usr/bin/env python3
""" HTTP Client

HTTP Client to make requests to Clover endpoints.
"""

import requests
import clover


class HTTPClient:

    @staticmethod
    def request(method, url, headers={}, params={}, **kwargs):
        """HTTP request."""

        # Headers.
        headers['Content-Type'] = 'application/json'
        headers['Authorization'] = 'Bearer {access_token}'.format(
            access_token=clover.access_token
        )

        # Query parameters.
        params['merchantUuid'] = clover.merchant_uuid
        params['mid'] = clover.mid


        return requests.request(
            method=method,
            url=url,
            headers=headers,
            params=params,
            **kwargs)

    @staticmethod
    def post(url, json, headers={}, params={}):
        """POST request."""
        
        return HTTPClient.request(
            method='POST',
            url=url,
            json=json,
            headers=headers,
            params=params)

    @staticmethod
    def get(url, headers={}, params={}):
        """GET request."""
        
        return HTTPClient.request(
            method='GET',
            url=url,
            headers=headers,
            params=params)

    @staticmethod
    def put(url, json, headers={}, params={}):
        """PUT request."""

        return HTTPClient.request(
            method='PUT',
            url=url,
            headers=headers,
            params=params,
            json=json)

    @staticmethod
    def delete(url, json, headers={}, params={}):
        """DELETE request."""

        return HTTPClient.request(
            method='DELETE',
            url=url,
            headers=headers,
            params=params,
            json=json)



#!/usr/bin/env python3
"""Mocked Test Case

Test case that will start mocking HTTP request methods.
"""
from os import environ
from clover.test import ApiTestCase, mock_request


DISABLE_MOCKING = False
if ('DISABLE_CLOVER_MOCKING' in environ and
    environ['DISABLE_CLOVER_MOCKING'] == 'true'):
    DISABLE_MOCKING = True

class MockedTestCase(ApiTestCase):
    
    @classmethod
    def setup_class(cls):
        """Starts mocks for HTTP requests."""

        if not DISABLE_MOCKING:
            # Create the patch object.
            cls.mock_request_patcher = mock_request('request')
            
            # Start the patch.
            cls.mock_request = cls.mock_request_patcher.start()

        super(MockedTestCase, cls).setup_class()

    @classmethod
    def teardown_class(cls):
        """Stops mocks for HTTP requests."""
        if not DISABLE_MOCKING:
            cls.mock_request_patcher.stop()

    def set_mock_json(self, data):
        """Sets the mock request json."""
        if not DISABLE_MOCKING:
            self.mock_request.return_value.json.return_value = data



#!/usr/bin/env python3
"""Charge Tests"""

from clover import Charge, Token
from clover.test import MockedTestCase
from clover.test.constants import CARD
from clover.utils import json_ps
from clover.test.mock_responses import (
    CREATE_TOKEN,
    CREATE_CHARGE,
    RETRIEVE_CHARGE,
    LIST_CHARGE
)


class TestCharge(MockedTestCase):
    
    def test_create_charge(self):
        """Trys to create a charge."""

        self.set_mock_json(CREATE_TOKEN)

        token = token = Token.create(
            card=CARD
        )

        self.set_mock_json(CREATE_CHARGE)

        charge = Charge.create(
            amount='999',
            currency='usd',
            description='Example charge',
            source=token['id'],
        )

        self.assertKeyValueEqual(charge, 'status', 'succeeded')

    def test_retrieve_charge(self):
        """Trys to retrieve a charge."""

        self.set_mock_json(CREATE_TOKEN)

        token = token = Token.create(
            card=CARD
        )

        self.set_mock_json(CREATE_CHARGE)

        charge = Charge.create(
            amount='999',
            currency='usd',
            description='Example charge',
            source=token['id'],
        )

        self.set_mock_json(RETRIEVE_CHARGE)

        r_charge = Charge.retrieve(
            id=charge['id']
        )

        # Check that it isn't an error code.
        self.assertNotInDict('error', r_charge, r_charge)

        # Check that it is the same charge object.
        self.assertEqual(r_charge['id'], charge['id'], json_ps(r_charge))

    def test_list_charge(self):
        """Tests listing charges."""

        self.set_mock_json(CREATE_TOKEN)

        token = token = Token.create(
            card=CARD
        )

        num_charges = 3

        # Make 3 charges so that they can be listed.
        for _ in range(num_charges):
            Charge.create(
                amount='999',
                currency='usd',
                description='Example charge',
                source=token['id'],
            )

        self.set_mock_json(LIST_CHARGE)

        charges = Charge.list(limit=num_charges)

        # Check that it isn't an error code.
        self.assertNotInDict('error', charges, charges)

        # Check that it is a list object.
        self.assertKeyValueEqual(charges, 'object', 'list')

        # Check that the correct amount was listed.
        self.assertEqual(
            len(charges['data']['elements']),
            num_charges,
            json_ps(charges)
        )
